@@ -2,10 +2,11 @@ import numpy as np
 import neworder as no
 import platform
 
+from utils import assert_throws
 
 def test_mc():
 
-  model = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+  model = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
 
   x = model.mc().ustream(1)
   model.mc().reset()
@@ -14,11 +15,12 @@ def test_mc():
   if no.mpi.size() == 1:
     _test_mc_serial(model)
   else:
-    _test_mc_parallel(model)
+    model_i = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_independent_stream)
+    _test_mc_parallel(model, model_i)
 
 def test_seeders():
 
-  # serial tests 
+  # serial tests
   # determinisitc seeders always return the same value
   assert no.MonteCarlo.deterministic_identical_stream(no.mpi.rank()) == no.MonteCarlo.deterministic_identical_stream(no.mpi.rank())
   assert no.MonteCarlo.deterministic_independent_stream(no.mpi.rank()) == no.MonteCarlo.deterministic_independent_stream(no.mpi.rank())
@@ -28,7 +30,7 @@ def test_seeders():
   try:
     import mpi4py.MPI as mpi
     comm = mpi.COMM_WORLD
-  except Exception: 
+  except Exception:
     return
 
   # parallel tests
@@ -60,56 +62,80 @@ def test_seeders():
 
   # test custom seeder
   seeder = lambda r: r + 1
-  m = no.Model(no.Timeline.null(), seeder)
+  m = no.Model(no.NoTimeline(), seeder)
   assert m.mc().seed() == no.mpi.rank() + 1
 
-def _assert_throws(f, e):
-  try:
-    f()
-  except e:
-    pass
-  else:
-    assert False, "expected exception %s not thrown" % e
 
 def test_sample():
-  m = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+  m = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
 
-  _assert_throws(lambda: m.mc().sample(100, [0.9]), ValueError)    
-  _assert_throws(lambda: m.mc().sample(100, [-0.1, 1.1]), ValueError)    
-  assert np.all(m.mc().sample(100, [1.0, 0.0, 0.0, 0.0]) == 0)    
-  assert np.all(m.mc().sample(100, [0.0, 1.0, 0.0, 0.0]) == 1)    
-  assert np.all(m.mc().sample(100, [0.0, 0.0, 0.0, 1.0]) == 3)    
+  assert_throws(ValueError, m.mc().sample, 100, [0.9])
+  assert_throws(ValueError, m.mc().sample, 100, [-0.1, 1.1])
+  assert np.all(m.mc().sample(100, [1.0, 0.0, 0.0, 0.0]) == 0)
+  assert np.all(m.mc().sample(100, [0.0, 1.0, 0.0, 0.0]) == 1)
+  assert np.all(m.mc().sample(100, [0.0, 0.0, 0.0, 1.0]) == 3)
 
 def test_hazard():
-  m = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+  m = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
 
   assert np.all(m.mc().hazard(0.0,10) == 0.0)
   assert np.all(m.mc().hazard(1.0,10) == 1.0)
 
-  _assert_throws(lambda: m.mc().hazard(-0.1, 10), ValueError)    
-  _assert_throws(lambda: m.mc().hazard( 1.1, 10), ValueError)    
-  _assert_throws(lambda: m.mc().hazard([-0.1, 0.5]), ValueError)    
-  _assert_throws(lambda: m.mc().hazard([0.1, 1.2]), ValueError)    
+  assert_throws(ValueError, m.mc().hazard, -0.1, 10)
+  assert_throws(ValueError, m.mc().hazard, 1.1, 10)
+  assert_throws(ValueError, m.mc().hazard, [-0.1, 0.5])
+  assert_throws(ValueError, m.mc().hazard, [0.1, 1.2])
+  assert_throws(ValueError, m.mc().hazard, np.nan, 1)
+  assert_throws(ValueError, m.mc().hazard, [0.1, np.nan])
 
 def test_stopping():
-  m = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+  m = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
 
   assert np.all(m.mc().stopping(0.0,10) == no.time.far_future())
 
-  _assert_throws(lambda: m.mc().stopping(-0.1, 10), ValueError)    
-  _assert_throws(lambda: m.mc().stopping( 1.1, 10), ValueError)    
-  _assert_throws(lambda: m.mc().stopping([-0.1, 0.5]), ValueError)    
-  _assert_throws(lambda: m.mc().stopping([0.1, 1.2]), ValueError)   
+  assert_throws(ValueError, m.mc().stopping, -0.1, 10)
+  assert_throws(ValueError, m.mc().stopping,  1.1, 10)
+  assert_throws(ValueError, m.mc().stopping, [-0.1, 0.5])
+  assert_throws(ValueError, m.mc().stopping, [0.1, 1.2])
+  assert_throws(ValueError, m.mc().stopping, np.nan, 1)
+  assert_throws(ValueError, m.mc().stopping, [0.1, np.nan])
 
 def test_arrivals_validation():
-  m = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+  m = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
   assert np.all(no.time.isnever(m.mc().first_arrival([0.0,0.0], 1.0, 10)))
-  _assert_throws(lambda: m.mc().first_arrival([-1.0,0.0], 1.0, 10), ValueError)    
-  assert np.all(no.time.isnever(m.mc().next_arrival(np.zeros(10), [0.0,0.0], 1.0)))
-  _assert_throws(lambda: m.mc().next_arrival(np.zeros(10), [-1.0,0.0], 1.0), ValueError)
+  assert_throws(ValueError, m.mc().first_arrival, np.array([-1.0,0.0]), 1.0, 10)
+  assert_throws(ValueError, m.mc().first_arrival, [1.0,np.nan], 1.0, 10)
 
-  _assert_throws(lambda: m.mc().arrivals([-1.0,0.0], 1.0, 10, 0.0), ValueError)
-  _assert_throws(lambda: m.mc().arrivals([1.0,1.0], 1.0, 10, 0.0), ValueError)
+  assert np.all(no.time.isnever(m.mc().next_arrival(np.zeros(10), [0.0,0.0], 1.0)))
+  assert_throws(ValueError, m.mc().next_arrival, np.zeros(10), [-1.0,0.0], 1.0)
+  assert_throws(ValueError, m.mc().next_arrival, np.zeros(10), [np.nan,np.nan], 1.0)
+
+  assert_throws(ValueError, m.mc().arrivals, [-1.0,0.0], 1.0, 10, 0.0)
+  assert_throws(ValueError, m.mc().arrivals, [1.0,1.0], 1.0, 10, 0.0)
+  assert_throws(ValueError, m.mc().arrivals, [np.nan,np.nan], 1.0, 10, 0.0)
+
+def test_mc_counts():
+  m = no.Model(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
+  mc = m.mc()
+  assert mc.seed() == 19937
+
+  def poisson_pdf(x, l):
+    y = np.exp(-l)
+    return np.array([l**k * y / np.math.factorial(k) for k in x])
+
+
+  tests = [(1.0, 1.0, 10000), (3.0, 0.5, 10000), (0.2, 2.0, 10000), (10.0, 1.0, 1000), (3.0, 1.0, 100000)]
+
+  for lam, dt, n in tests:
+
+    c = mc.counts([lam] * n, dt)
+    x = range(0,max(c))
+    # convert to counts
+    c = [(c == k).sum() / n for k in x]
+    p = poisson_pdf(x, lam*dt)
+
+    for i in x:
+      assert np.fabs(c[i] - p[i]) < 1.0 / np.sqrt(n)
 
 def _test_mc_serial(model):
   mc = model.mc()
@@ -119,7 +145,7 @@ def _test_mc_serial(model):
   s = mc.state()
   a = mc.ustream(5)
   if platform.system() != "Darwin":
-    # mc.state() just returns 0 on OSX due to an apparent bug in MT19937 that intermittently segfaults 
+    # mc.state() just returns 0 on OSX due to an apparent bug in MT19937 that intermittently segfaults
     assert s != mc.state()
   assert abs(a[0] - 0.33778882725164294) < 1e-8
   assert abs(a[1] - 0.04767065867781639) < 1e-8
@@ -224,11 +250,12 @@ def _test_mc_serial(model):
   assert len(nhpp) == 10
 
 
-def _test_mc_parallel(model):
+def _test_mc_parallel(model, model_i):
 
   import mpi4py.MPI as mpi
   comm = mpi.COMM_WORLD
 
+  # test model has identical streams
   mc = model.mc()
   mc.reset()
   assert mc.seed() == 19937
@@ -242,19 +269,20 @@ def _test_mc_parallel(model):
       assert np.all(all_states[0] == all_states[r])
       assert np.all(a - all_a[r] == 0.0)
 
-  mc = no.Model(no.Timeline.null(), no.MonteCarlo.deterministic_independent_stream).mc()
+  # test model_i has independent streams
+  mc = model_i.mc()
+  mc.reset()
+  assert mc.seed() == 19937 + no.mpi.rank()
 
   a = mc.ustream(5)
-
   all_a = comm.gather(a, root=0)
-
   all_states = comm.gather(mc.state(), root=0)
 
   # check all other streams different
   if no.mpi.rank() == 0:
     for r in range(1, no.mpi.size()):
       if platform.system() != "Darwin":
-        # mc.state() just returns 0 on OSX due to an apparent bug in MT19937 that intermittently segfaults 
+        # mc.state() just returns 0 on OSX due to an apparent bug in MT19937 that intermittently segfaults
         assert all_states[r] != all_states[0]
       assert not np.all(a - all_a[r] == 0.0)
 
