@@ -1,8 +1,10 @@
 import pytest
 import numpy as np
-import neworder as no
 
+import warnings
+warnings.filterwarnings(action='ignore', category=RuntimeWarning, message=r't=')
 #no.verbose()
+import neworder as no
 
 
 def test_basics():
@@ -27,24 +29,23 @@ def test_submodules():
 def test_dummy_model():
   class DummyModel(no.Model):
     def __init__(self):
-      super().__init__(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+      super().__init__(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
     def step(self):
       pass
-    def checkpoint(self):
+    def finalise(self):
       pass
 
   assert no.run(DummyModel())
 
+@pytest.mark.filterwarnings("ignore:check()")
 def test_check_flag():
   class FailingModel(no.Model):
     def __init__(self):
-      super().__init__(no.Timeline.null(), no.MonteCarlo.deterministic_identical_stream)
+      super().__init__(no.NoTimeline(), no.MonteCarlo.deterministic_identical_stream)
     def step(self):
       pass
     def check(self):
       return False
-    def checkpoint(self):
-      pass
 
   # fails
   assert not no.run(FailingModel())
